import requests
from requests import Request, Response


def send(req: Request) -> Response:
  session = requests.Session()
  prepared = session.prepare_request(req)
  response = session.send(prepared)

  return response


from mmragsdk._requests.factory import  get_request
from mmragsdk import URL
from dotenv import load_dotenv
import os
from unittest.mock import ANY


TEST_API = "1234567890"
TEST_INPUT = 'user123'


def test_get_chat_request():
  request = get_request('chat', TEST_API, TEST_INPUT)

  assert request.url == URL + '/chat'
  assert request.method == "POST"
  assert request.headers == {"Content-Type": "application/json", "Authorization": "Bearer " + TEST_API}
  assert request.json == {"query": TEST_INPUT}

def test_get_search_request():
  request = get_request('search', TEST_API, TEST_INPUT)

  assert request.url == URL + '/search'
  assert request.method == "POST"
  assert request.headers == {"Content-Type": "application/json", "Authorization": "Bearer " + TEST_API}
  assert request.json == {"query": TEST_INPUT}


def test_get_upload_request():
  test_path = 'test.txt'
  with open(test_path, 'w') as f:
    f.write('test')

  request = get_request('upload', TEST_API, test_path)

  assert request.url == URL + '/upload-file'
  assert request.method == "POST"
  assert request.headers == {"Authorization": "Bearer " + TEST_API}
  assert request.files == {"file": (test_path, ANY, "multipart/form-data")}


def test_get_clean_request():
  request = get_request('clean', TEST_API, TEST_INPUT)

  assert request.url == URL + '/cleanUp'
  assert request.method == "POST"
  assert request.headers == {"Content-Type": "application/json", "Authorization": "Bearer " + TEST_API}
  assert request.json == {}



import unittest

from mmragsdk import Client
from dotenv import load_dotenv
import os

import pytest


load_dotenv()


RANDOM_PATH = 'random_path'
client = Client(token=os.getenv("TOKEN_PAT"))


def test_missing_file_path_error():
  with pytest.raises(FileNotFoundError):
    client.upload(RANDOM_PATH)


def test_empty_search_query_raises():
  with pytest.raises(ValueError):
    client.search('')


def test_empty_chat_prompt_raises():
  with pytest.raises(ValueError):
    client.chat('')


def test_chat_success():
  response = client.chat("How are you?")

  assert response.status_code == 200


def test_search_success():
  response = client.search('test')

  assert response.status_code == 200


def test_upload_success():
  path = 'test.txt'
  with open(path, 'w') as f:
    f.write('test')
  response = client.upload(path)

  assert response.status_code == 200

  if os.path.exists(path):
    os.remove(path)


def test_clean_success():
  response = client.clean()
  assert response.status_code == 200


def test_invalid_token_chat_error():
  client.token = 'wrong token'
  response = client.chat('t')
  assert response.status_code == 403


def test_invalid_token_search_error():
  client.token = 'wrong token'
  response = client.search('t')
  assert response.status_code == 403


def test_invalid_token_upload_error():
  client.token = 'wrong token'
  response = client.chat('t')
  assert response.status_code == 403


def test_invalid_token_clean_error():
  client.token = 'wrong token'
  response = client.clean()
  assert response.status_code == 403



import os

from requests import Request


URL = "https://0jl0v93oi5.execute-api.eu-central-1.amazonaws.com/"


def get_request(function_name: str, api: str, user_input: str) -> Request:
  if function_name == "chat":
    return Request(
      "POST",
      url=URL + '/chat',
      headers={
        "Content-Type": "application/json",
        "Authorization": f"Bearer {api}"
      },
      json={
        "query": user_input,
      }
    )
  elif function_name == "search":
    return Request(
      "POST",
      url=URL + '/search',
      headers={
        "Content-Type": "application/json",
        "Authorization": f"Bearer {api}"
      },
      json={
        "query": user_input,
      }
    )
  elif function_name == "upload":
    return Request(
      "POST",
      url=URL + '/upload-file',
      headers={
        "Authorization": f"Bearer {api}"
      },
      files={
        "file": (os.path.basename(user_input), open(user_input), "multipart/form-data")
      }
    )
  elif function_name == "clean":
    return Request(
      "POST",
      url=URL + '/cleanUp',
      headers={
        "Content-Type": "application/json",
        "Authorization": f"Bearer {api}"
      },
      json={}
    )
  else:
    raise NotImplementedError(
      f"Function {function_name} not implemented"
    )


import os
from dotenv import load_dotenv

from pydantic import BaseModel
from mmragsdk._requests import factory, send
from requests import Response


load_dotenv()


class Client(BaseModel):
  token: str

  def chat(self, prompt: str) -> Response:
    if len(prompt) == 0:
      raise ValueError(
        f"Cannot chat with an empty query: {prompt}"
      )
    request = factory.get_request(self.chat.__name__, self.token, prompt)
    return send(request)

  def search(self, query: str) -> Response:
    if len(query) == 0:
      raise ValueError(
        f"Cannot search with an empty query: {query}"
      )

    request = factory.get_request(self.search.__name__, self.token, query)
    return send(request)

  def upload(self, file_path: str) -> Response:
    if not os.path.exists(file_path):
      raise FileNotFoundError(
        f"Unable to locate the file {file_path} on the system, please check for misspellings"
      )
    request = factory.get_request(self.upload.__name__, self.token, file_path)
    return send(request)

  def clean(self) -> Response:
    request = factory.get_request(self.clean.__name__, self.token, '')
    return send(request)


if __name__ == "__main__":
  client = Client(
    token=os.getenv("TOKEN_PAT"),
  )

  # response = client.chat(prompt="What is your name?")
  # print(response.json())
  #
  # with open("test.txt", "w") as f:
  #   f.write('this is a test')
  #
  # res = client.upload("test.txt")
  # print(res.json())
  #
  # res = client.search("is this a test")
  # print(res.json())

  res = client.clean()
  print(res)







from pydantic import BaseModel


class Request(BaseModel):
  url: str


class SearchRequest(Request):
  pass


class UploadRequest(Request):
  pass


class ChatRequest(Request):
  pass


class CleanRequest(Request):
  pass
